import argparse
import json
import os
import shutil
import subprocess


# import docker


def is_supported_type(identifier):
    not_supported = ['array', 'contract', 'enum', 'function_external', 'struct', 'userDefinedValue']
    return not True in [n in identifier for n in not_supported]


class SolParser:
    @classmethod
    def read(self, file):
        if os.path.splitext(file)[1] == ".json":
            f = open(file, "r")
            data = json.load(f)
            # print(data)
            return data
        if os.path.splitext(file)[1] == ".sol":
            command = ['forge', 'flatten', '--output', 'tmp.sol', file]
            subprocess.run(command)
            file_exists = os.path.exists('tmp.sol')

            if not file_exists:
                shutil.copyfile(file, './tmp.sol')

            with open('tmp.sol', 'r') as f:
                lines = f.readlines()

            filtered_lines = [line for line in lines if
                              not line.replace(" ", "").startswith('*') and not line.replace(" ", "").startswith('/**') and not line.replace(" ", "").startswith('/*')]
            with open('tmp.sol', 'w') as f:
                f.writelines(filtered_lines)
            print("FILTERED: ", len(filtered_lines))
            print("NON FILTERED: ", len(lines))
            command = ['solc', 'tmp.sol', '--ast-compact-json']
            # command = ['docker', 'run', '-v', '/Users/konstantin.britikov/Documents/SMT/rand_proj/script:/script/', 'ethereum/solc:0.6.12', "/script/" + file.split("/")[-1], '--ast-compact-json']
            print(command)
            # exit(1)
            with subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE) as process:
                try:
                    stdout, stderr = process.communicate(input, timeout=20)
                    print("Error: ", stderr)
                    stdout_results = stdout.decode("utf-8")
                    stdout_results = stdout_results.replace("\'", "\"")
                    # print("Stdout results: ", stdout_results)
                    next_line = False
                    index = 0
                    print(file)
                    filename = file.split("/")[-1]
                    # print("File: ", file)
                    for i, line in enumerate(stdout_results.split('\n')):
                        print("Iterators: ", i, " : ", line)
                        if next_line:
                            break
                        if filename in line:
                            next_line = True
                            index = i
                    split_arr = stdout_results.split('\n')

                    # print("Splitted: ", split_arr)
                    # print("Splitted len: ", len(split_arr))
                    cut_result = '\n'.join(stdout_results.split('\n')[index + 1:])
                    cut_result = split_arr[-1]
                    # print("Cut result: ", cut_result)
                    # print("Index: ", index)
                    # exit(1)
                    print("CUT: ",stdout_results)
                    data = json.loads(cut_result)
                    # print(data)
                    return data
                except subprocess.TimeoutExpired:
                    process.kill()
                    mesage = 'command: {} has been killed after timeout {}'.format("solc", 20)
                    print(mesage)
                    return []

    @classmethod
    def parse_data(self, data):
        n_of_contracts = len(data['nodes'])
        print("number of contracts: {}".format(n_of_contracts))
        out = []
        
        contracts_from_id = {}
        for n in data['nodes']:
            if n['nodeType'] != "ContractDefinition":
                continue
            id = n['id']
            contracts_from_id[id] = n

        for n in data['nodes']:
            if n['nodeType'] != "ContractDefinition":
                continue
            for_one_contract = []
            name = n['name']
            contract_id = n['id']
            print("name: {} id: {}".format(name, contract_id))
            contractKind = n['contractKind']
            print("Abstract:", n['abstract'])
            if contractKind != "contract" or n['abstract'] == 'True':
                print("Parsing stopped")
                continue
            print("contractKind: {}".format(contractKind))
            p_type1 = 'state_type'
            p_name1 = 'state'
            p_type2 = 'uint'
            p_name2 = 'msg.value'
            p_type3 = 'address'
            p_name3 = 'msg.sender'
            for_one_contract.append([name, contractKind, contract_id, p_type1, p_name1, p_type2, p_name2, p_type3, p_name3])
            print(for_one_contract)
            f_c = n['nodes']
            # print("Node: ", f_c)
            f_c = []
            for basecontract_id in n["linearizedBaseContracts"]:
                f_c = f_c + contracts_from_id[basecontract_id]['nodes']
            
            selectors_found = []
            for fc in f_c:
                if 'kind' not in fc:
                    continue
                selector = ''
                try:
                    selector = fc["functionSelector"]
                except:
                    print('error functionselector\n')
                    print(fc)
                    exit
                
                if selector in selectors_found:
                    continue
                selectors_found.append(selector)
                
                f_name = fc['name']
                f_kind = fc['kind']
                f_id = fc['id']
                f_mutability = fc['stateMutability']
                print("Kind:", f_kind)
                print("Name:", f_name)
                print("Id:", f_id)
                print("Mutability:", f_mutability)
                if f_name == "deposit":
                    print(fc)
                if f_kind == 'constructor':
                    print(f_name)
                    print("Constructor: ", fc)
                    parameters = fc['parameters']['parameters']
                    constructor_parameters = []
                    for p in parameters:
                        identifier = p["typeDescriptions"]["typeIdentifier"]
                        if is_supported_type(identifier):
                            constructor_parameters.append(p['typeName']['name'])
                            constructor_parameters.append(p['name'])
                    print("Constructor parameters: ", constructor_parameters)
                    # find [name, contractKind] and add constructor_parameters
                    print("For one con before: ", for_one_contract)
                    for el in for_one_contract:
                        if el[0] == name and el[1] == 'contract':
                            el += constructor_parameters
                    print("For one con after: ", for_one_contract)
                    continue
                elif (f_kind == 'function' or f_kind == 'fallback' or f_kind == 'receive') and (
                        fc['visibility'] == 'public' or fc['visibility'] == 'external'):
                    tmp_f = [f_name, f_id]
                    print("FUNCTION:")
                    print("kind: {} name: {} id: {}".format(f_kind, f_name, f_id))
                    params = fc['parameters']['parameters']
                    print("# of parameters: {}".format(len(params)))
                    # if f_mutability == 'payable':
                    #     identifier = 'msg'
                    #     p_type = 'tx_type'
                    #     p_name = 'tx'
                    #     tmp_f.append(p_type)
                    #     tmp_f.append(p_name)
                    p_type = 'state_type'
                    p_name = 'state'
                    tmp_f.append(p_type)
                    tmp_f.append(p_name)
                    p_type = 'uint'
                    p_name = 'msg.value'
                    tmp_f.append(p_type)
                    tmp_f.append(p_name)
                    p_type = 'address'
                    p_name = 'msg.sender'
                    tmp_f.append(p_type)
                    tmp_f.append(p_name)
                    # p_type = 'uint'
                    # p_name = 'balanceOf'
                    # tmp_f.append(p_type)
                    # tmp_f.append(p_name)
                    for p in params:
                        identifier = p["typeDescriptions"]["typeIdentifier"]
                        if is_supported_type(identifier):
                            p_type = p['typeName']['name']
                            p_name = p['name']
                            tmp_f.append(p_type)
                            tmp_f.append(p_name)
                        elif 'contract' in identifier:
                            tmp = p['typeDescriptions']['typeString']
                            if 'contract' in tmp:
                                p_type = tmp.split()[1]
                                p_name = p['name']
                            else:
                                continue  # unkown ow type
                        elif not is_supported_type(identifier):
                            tmp_f = []  # enum case or function_external or struct : skip
                        else:
                            if 'name' in p['typeName']['baseType']:
                                p_type = p['typeName']['baseType']['name']
                                p_name = p['name']
                                tmp_f.append(p_type + "[]")
                                tmp_f.append(p_name)
                            else:
                                tmp_f = []  # case of array[][] not supported currently
                                continue
                        # print("type: {}".format(p_type))
                        # print("name: {}".format(p_name))
                    if tmp_f:
                        for_one_contract.append(tmp_f)
                elif f_kind == 'function' and (fc['visibility'] == 'private' or fc['visibility'] == 'internal'):
                    continue
                else:
                    print("SOMETHING WEIRD")
                    print(f_kind)
                    print(fc['visibility'])
                    exit(1)
            if len(for_one_contract) > 1:
                out.append(for_one_contract)
        # exit(1)
        return out

    @classmethod
    def get_signature(self, file):
        d = SolParser.read(file)
        if d:
            return SolParser.parse_data(d)
        else:
            return []


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='python script for sol file parse')
    insourse = ['-i', '--input_file']
    kwsourse = {'type': str, 'help': 'path: location of the file'}
    parser.add_argument(*insourse, **kwsourse)
    args = parser.parse_args()

    if args.input_file is not None and os.path.isfile(args.input_file):
        f = args.input_file
        print('input file {}'.format(f))
    else:
        exit(1)

    d = SolParser.read(f)
    if d:
        print(SolParser.parse_data(d))



import argparse
import os
import re

import xlsxwriter


class html_report:

    def create_header(table):
        header_line = "No., Sourse dir, Sourse code, Links (smt2 logs reports) , Tests, Coverage, Time"
        header = header_line.split(",")
        table += "  <tr>\n"
        for column in header:
            table += "    <th>{0}</th>\n".format(column.strip())
        table += "  </tr>\n"
        return table


    def create_hyperlinnk_to_file(text):
        if not text:
            return "NaN"
        name = os.path.basename(text)
        if os.path.exists(text):
            return "<a href=\"{0}\">{1} </a>\n".format(text, name)
        else:
            return "NaN"

    def create_hyperlinnk_to_test_file(text):
        if not text:
            return "NaN"
        if os.path.exists(text):
            return "<a href=\"{0}\">{1} </a>\n".format(text, "test")
        else:
            return "NaN"

    def smt2_status(ll):
        if "smt2" not in ll:
            return "-"
        else:
            return ll[13]

    def smt2_number_of_lines(smt2file):
        if os.path.exists(smt2file):
            return len(open(smt2file).readlines())
        else:
            return "-"

    def link_to_log(p):
        name = os.path.dirname(p) + "/log.txt"
        return "<a href=\"{0}\">{1}</a>\n".format(name, "log")

    def get_z3_results(p):
        if "z3_error" in p:
            return "timeout"
        if len(p) > 15:
            if ('sat' in p[16] and 'unsat' not in p[16]):
                return 'sat'
            if ('unsat' in p[16]):
                return 'unsat'
        else:
            return "-"

    def parse_result_line(line):
        tmp = line.split(';')
        tmp_1 = tmp[0].split()
        out = ''
        if tmp_1[-2].isnumeric():
            if int(tmp_1[-2]) > 0:
               out += "<br/><font color=green>{}</font> ".format(tmp_1[-2] + " " + tmp_1[-1])
        tmp_2 = tmp[1].split()
        if tmp_2[-2].isnumeric():
            if int(tmp_2[-2]) > 0:
                out += "<br/><font color=red>{}</font> ".format(tmp[1])
        return out

    def parse_fun(line):
        tmp = line.split(';')
        tmp_1 = tmp[0].split()
        out = ''
        if tmp_1[-2].isnumeric():
            if int(tmp_1[-2]) > 0:
               out += tmp_1[-2] + " " + tmp_1[-1]
        tmp_2 = tmp[1].split()
        if tmp_2[-2].isnumeric():
            if int(tmp_2[-2]) > 0:
                out += tmp[1]
        return out

    @classmethod
    def get_extra_info_from_log(cls, dir):
        log = [f.path for f in os.scandir(dir) if f.is_file() and os.path.basename(f) == 'log.txt']
        out = ''
        if len(log) >= 1:
            what_to_check = ["Multiple queries are not supported",
                             "Assertion failed",
                             "Done with TG",
                             "array operation requires one sort parameter",
                             "ALL Branches are covered: DONE",
                             "FOUND", 'unrolling sat', 'unrolling unsat',
                             "index_cycle_chc :",
                             'RUST_BACKTRACE',
                             "# TESTS:", 'Multiple Calls Test']
            filein = open(log[0], "r", encoding='ISO-8859-1')
            lines = filein.readlines()
            for w in what_to_check:
                for line in lines:
                    if re.search(w, line):
                        out += "<br/>" + "<font color=8B008B>{}</font>\n".format(line)
                        break
        return out



    @classmethod
    def clear_benchmarkdir(self, dir, nonlinear):
        for n in nonlinear:
            tmp = dir + "/" + n + ".c"
            if os.path.isfile(tmp):
                # remove dir
                os.remove(tmp)


    @classmethod
    def get_tests_info(cls, dir):
        test_results = [f.path for f in os.scandir(dir) if f.is_file() and os.path.basename(f) == 'test_results.txt']
        out = ''
        if len(test_results) >= 1:
            what_to_check = ["No tests match",
                             "Unnamed return variable",
                             "Done with TG",
                             "array operation requires one sort parameter"]
            filein = open(test_results[0], "r", encoding='ISO-8859-1')
            lines = filein.readlines()
            for line in lines:
                if "Test result:" in line:
                    out += "<br/>" + cls.parse_result_line(line)
            for w in what_to_check:
                for line in lines:
                    if re.search(w, line):
                        out += "<br/>" + "<font color=8B008B>{}</font>\n".format(w)
                        break
            return out
        else:
            return "No info"


    @classmethod
    def get_number_of_test(cls, dir):
        test_results = [f.path for f in os.scandir(dir) if f.is_file() and os.path.basename(f) == 'test_results.txt']
        out = 0
        if len(test_results) >= 1:
            filein = open(test_results[0], "r", encoding='ISO-8859-1')
            lines = filein.readlines()
            for line in lines:
                if "Test result:" in line:
                    tmp = cls.parse_fun(line).split()
                    if len(tmp) > 1:
                        out += int(tmp[0])
            return out
        else:
            return 0


    @classmethod
    def get_number_of_line_in_original_sorse_file(cls, dir):
        sol_file = dir + "/" + os.path.basename(dir) + ".sol"
        if os.path.exists(sol_file):
            return len(open(sol_file, "r").readlines())
        else:
            return 0


    @classmethod
    def read_lcov_html_report(cls, file_name):
        file = open(file_name, "r")
        lines = file.readlines()
        brench_lines = []
        flag = False
        i = 0
        for line in lines:
            if flag:
                brench_lines.append(re.sub('<[^<]+?>', '', line))
                i += 1
            if 'Branches:' in line:
                brench_lines.append(re.sub('<[^<]+?>', '', line))
                flag = True
            if i == 3:
                break
        return '{}<br/>\nHit: {}<br/>\nTotal: {}<br/>\nCoverage: {}\n'.format(brench_lines[0], brench_lines[1],
                                                                              brench_lines[2], brench_lines[3])



    @classmethod
    def get_coverage_data(cls, dir):
        sub_dirs = [f.path for f in os.scandir(dir) if f.is_dir() and os.path.basename(f) in 'generated-coverage']
        if len(sub_dirs) != 1:
            return "<font color=\"red\">{}</font>\n".format('no data')
        else:
            report_dir = [f.path for f in os.scandir(sub_dirs[0]) if f.is_dir()]
            report_dir = [d for d in report_dir if os.path.basename(d)]
            if len(report_dir) != 1:
                return "<font color=\"red\">{}</font>\n".format('no report')
            else:
                file_name = report_dir[0] + '/src/' + os.path.basename(dir) +'.sol.gcov.html'
                out = "<a href=\"{0}\">{1} </a>\n".format(file_name, "coverage_c_file_TG") + '<br/>\n'
                out += html_report.read_lcov_html_report(file_name) + '<br/>'
                return out


    @classmethod
    def read_lcov_html_report_plane_text(cls, file_name):
        file = open(file_name, "r")
        lines = file.readlines()
        brench_lines = []
        flag = False
        i = 0
        for line in lines:
            if flag:
                brench_lines.append(re.sub('<[^<]+?>', '', line))
                i += 1
            if 'Branches:' in line:
                brench_lines.append(re.sub('<[^<]+?>', '', line))
                flag = True
            if i == 3:
                break
        return brench_lines



    @classmethod
    def get_coverage_data_plane_text(cls, dir):
        sub_dirs = [f.path for f in os.scandir(dir) if f.is_dir() and os.path.basename(f) in 'generated-coverage']
        if len(sub_dirs) != 1:
            return 'no data'
        else:
            report_dir = [f.path for f in os.scandir(sub_dirs[0]) if f.is_dir()]
            report_dir = [d for d in report_dir if os.path.basename(d)]
            if len(report_dir) != 1:
                return 'no report'
            else:
                file_name = report_dir[0] + '/src/' + os.path.basename(dir) +'.sol.gcov.html'
                out = html_report.read_lcov_html_report_plane_text(file_name)
                return out


    @classmethod
    def read_lcov_html_report_plane_text_function_number(cls, file_name):
        file = open(file_name, "r")
        lines = file.readlines()
        brench_lines = []
        flag = False
        i = 0

        for line in lines:
            if flag:
                brench_lines.append(re.sub('<[^<]+?>', '', line))
                i += 1
            if 'Functions:' in line:
                brench_lines.append(re.sub('<[^<]+?>', '', line))
                flag = True
            if i == 3:
                break
        return brench_lines


    @classmethod
    def get_function_number_plane_text(cls, dir):
        sub_dirs = [f.path for f in os.scandir(dir) if f.is_dir() and os.path.basename(f) in 'generated-coverage']
        if len(sub_dirs) != 1:
            return 'no data'
        else:
            report_dir = [f.path for f in os.scandir(sub_dirs[0]) if f.is_dir()]
            exclude = ['usr']
            report_dir = [d for d in report_dir if os.path.basename(d) not in exclude]
            if len(report_dir) != 1:
                return 'no report'
            else:
                file_name = report_dir[0] + '/src/' + os.path.basename(dir) +'.sol.gcov.html'
                out = html_report.read_lcov_html_report_plane_text_function_number(file_name)
                return out



    @classmethod
    def buildReport(self, dir):
        fileout = open("{}/1_html_report.html".format(dir), "w+")

        table = "<table border=\"1\" cellspacing=\"0\" cellpadding=\"4\">\n"
        table = html_report.create_header(table)

        i = 1
        subdirs = [f.path for f in os.scandir(dir) if f.is_dir() and os.path.basename(f)]
        out = []
        for s in subdirs:
            out += [(s, f.path) for f in os.scandir(s) if f.is_dir() and os.path.basename(f)]
        for o in sorted(out):
            (subd, line) = o
            print(line)
            table += "  <tr>\n"
            table += "    <td>{0}</td>\n".format(i)
            table += "    <td>{0}<br/>\n".format(
                html_report.create_hyperlinnk_to_file(subd))
            table += "    <td>{0}<br/>\n".format(
                html_report.create_hyperlinnk_to_file(line + '/' + os.path.basename(line) + '.sol'))
            table += "    <td>{0}<br/>{1}<br/>{2}<br/>{3}<br/>{4}<br/></td>\n".format(html_report.get_smt2_file(line),
                                                            html_report.get_log_file(line, "log.txt"),
                                                            html_report.get_log_file(line, "log_encoding.txt"),
                                                            html_report.get_extra_info_from_log(line),
                                                            html_report.get_log_file(line, "imag.png"))
            table += "    <td>{0}<br/>{1}<br/>{2}</br>{3}</td>\n".format(html_report.create_hyperlinnk_to_test_file(line + '/' + os.path.basename(line) + '.t.sol'),
                                                         html_report.get_log_file(line, "test_results.txt"),
                                                        html_report.create_hyperlinnk_to_file(line + '/testgen.txt'),
                                                        html_report.get_tests_info(line))
            table += "    <td>{0}</td>\n".format(html_report.get_coverage_data(line))
            table += "    <td>{0}</td>\n".format(str(html_report.get_time_consumed(line)) + ' seconds')
            table += "  </tr>\n"
            i += 1
        table += "</table>"
        # table = table.replace("../{}".format(dir), ".")
        #table = table.replace(dir, ".")
        fileout.writelines(table)
        fileout.close()



    @classmethod
    def get_smt2_file(cls, dir):
        smt2files = [f.path for f in os.scandir(dir) if f.is_file() and os.path.splitext(f)[1] == '.smt2']
        if len(smt2files) >= 1:
            out = ""
            for f in sorted(smt2files):
                if "wo_adt" not in f:
                    out += "<font color=\"black\">[original smt]<br/> {}</font><br/>\n".format(html_report.create_hyperlinnk_to_file(f))
                else:
                    out += "<font color=\"black\">[adt free smt]<br/> {}</font><br/>\n".format(
                        html_report.create_hyperlinnk_to_file(f))
            return out
        else:
            return "<font color=\"red\">{}</font>\n".format('no smt')

    @classmethod
    def get_log_file(cls, dir, log_file_name):
        log = [f.path for f in os.scandir(dir) if f.is_file() and os.path.basename(f) == log_file_name]
        if len(log) >= 1:
            return html_report.create_hyperlinnk_to_file(log[0])
        else:
            return "<font color=\"red\">{}</font>\n".format('no log')


    @classmethod
    def get_time_consumed(cls, dir):
        log = [f.path for f in os.scandir(dir) if f.is_file() and os.path.basename(f) == 'log.txt']
        if len(log) >= 1:
            with open(log[0], 'rb') as f:
                f.seek(-2, os.SEEK_END)
                while f.read(1) != b'\n':
                    f.seek(-2, os.SEEK_CUR)
                last_line = f.readline().decode()
                time_con = last_line.split()
                if len(time_con) > 3 and ("total time" in last_line):
                    return "%8.2f" % (float(time_con[2]))
                else:
                    "<font color=\"red\">{}</font>\n".format('no available')
        else:
            return "<font color=\"red\">{}</font>\n".format('no available')



    def is_nonlinear(name):
        filein = open(name, "r", encoding='ISO-8859-1')
        lines = filein.readlines()
        for l in lines:
            if "Nonlinear CHC is currently unsupported" in l:
                return False
        return True


    @classmethod
    def build_excel_report(self, dir):
        # Create a workbook and add a worksheet.
        workbook = xlsxwriter.Workbook(dir + '/1_report.xlsx')
        worksheet = workbook.add_worksheet()

        expenses = [['dir', 'filename', 'coverage',  'time', 'hit', 'total', '#fun_hit', '#fun_total', '# tests', '# lines']]
        subdirs = [f.path for f in os.scandir(dir) if f.is_dir() and os.path.basename(f)]
        out = []
        for s in subdirs:
            out += [(s, f.path) for f in os.scandir(s) if f.is_dir() and os.path.basename(f)]
        for o in sorted(out):
            (subd, line) = o
            dir_name = os.path.basename(subd)
            file_name = os.path.basename(line) + '.sol'
            raw_data = html_report.get_coverage_data_plane_text(line)
            coverage = ""
            if raw_data != "no data" and raw_data != 'no report':
                raw_data = [r.strip("\n") for r in raw_data]
                coverage = raw_data[1].strip("%")
                print(raw_data)
                hit = float(raw_data[3])
                total = float(raw_data[2])
            else:
                coverage = 0
                hit = ''
                total = ''

            fun_number = html_report.get_function_number_plane_text(line)
            if not isinstance(fun_number, (bool)) and fun_number != "no data" and raw_data != 'no report':
                fun_number = [r.strip("\n") for r in fun_number]
                test1 = fun_number[1]
                test2 = fun_number[2]
            else:
                coverage_TG = 0
                test1 = ''
                test2 = ''

            time = html_report.get_time_consumed(line)
            number_of_tests = html_report.get_number_of_test(line)
            number_of_line = html_report.get_number_of_line_in_original_sorse_file(line)
            expenses.append([dir_name, file_name, coverage, time, hit, total, test1, test2, number_of_tests, number_of_line])

        row = 0
        col = 0

        for dir_name, file_name, coverage, time, hit, total, fun_number_hit, fun_number_total, number_of_tests, number_of_line in expenses:
            worksheet.write(row, col, dir_name)
            worksheet.write(row, col + 1, file_name)
            worksheet.write(row, col + 2, coverage)
            worksheet.write(row, col + 3, time)
            worksheet.write(row, col + 4, hit)
            worksheet.write(row, col + 5, total)
            worksheet.write(row, col + 6, fun_number_hit)
            worksheet.write(row, col + 7, fun_number_total)
            worksheet.write(row, col + 8, number_of_tests)
            worksheet.write(row, col + 9, number_of_line)
            row += 1

        workbook.close()



if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='python script for Report Builder')
    insourse = ['-i', '--input_dir']
    kwsourse = {'type': str, 'help': 'dir: where TG run is located'}
    parser.add_argument(*insourse, **kwsourse)
    args = parser.parse_args()

    if args.input_dir is not None:
        if os.path.isdir(args.input_dir):
            dir = args.input_dir
            print('report dir set to {}'.format(dir))
    else:
        dir = "/Users/ilyazlatkin/CLionProjects/blockchain_exp/hello_foundry/testgen_output"
        dir = "/Users/ilyazlatkin/Downloads/testgen_output"
        #dir = "/Users/ilyazlatkin/PycharmProjects/results/blockchain/regression_sanity_2/testgen_output"

    html_report.buildReport(dir)
    html_report.build_excel_report(dir)




import argparse
import time
from datetime import datetime
import os
import shutil
import subprocess
import solTg.SolidityTestGen as SolidityTestGen
from solTg.ReportBuilder import html_report

""" Tools location
"""


def init():
    global SOURCE_PATH, SANDBOX_DIR, OUTPUTDIR, TIMEOUT
    # tmp = os.path.dirname(os.path.dirname((os.path.dirname(os.path.realpath(__file__)))))
    tmp = os.getcwd()
    SANDBOX_DIR = tmp + "/sandbox"
    OUTPUTDIR = tmp + "/test"
    TIMEOUT = 120

def clean_dir(dir):
    for root, dirs, files in os.walk(dir):
        for f in files:
            os.unlink(os.path.join(root, f))
        for d in dirs:
            shutil.rmtree(os.path.join(root, d))
    if not os.path.exists(dir):
        os.mkdir(dir)


def copy_dir(root_src_dir, root_dst_dir):
    for src_dir, dirs, files in os.walk(root_src_dir):
        dst_dir = src_dir.replace(root_src_dir, root_dst_dir, 1)
        if not os.path.exists(dst_dir):
            os.makedirs(dst_dir)
        for file_ in files:
            src_file = os.path.join(src_dir, file_)
            dst_file = os.path.join(dst_dir, file_)
            if os.path.exists(dst_file):
                os.remove(dst_file)
            shutil.copy(src_file, dst_dir)


def move_to_sandbox(files):
    print("========move_to_sandbox===========")
    if not os.path.exists(SANDBOX_DIR):
        os.mkdir(SANDBOX_DIR)
    else:
        print('clear output directory {}'.format(SANDBOX_DIR))
        # remove dir
        os_info = os.uname()
        if (os_info.sysname != 'Darwin'):
            clean_dir(SANDBOX_DIR)
        else:
            shutil.rmtree(SANDBOX_DIR)
            os.mkdir(SANDBOX_DIR)
    new_file_list = []
    for f in files:
        # create subdir for each .c file
        basename = os.path.basename(f)
        name_wo_ext = os.path.splitext(basename)[0]
        subdir = SANDBOX_DIR + "/" + name_wo_ext
        os.mkdir(subdir)
        # copy file to individual sandbox
        new_file = subdir + "/" + basename
        shutil.copyfile(f, new_file)
        new_file_list.append(new_file)
    return new_file_list



def logger(file, content):
    f = open(file, 'a')
    now = datetime.now()
    time = now.strftime("%H:%M:%S:%f")
    t = str('[{}]'.format(time))
    if type(content) is list:
        f.writelines([t] + ['\n'])
        for c in content:
            if type(c) is list:
                f.writelines([' '.join(c)] + ['\n'])
            elif type(c) is bytes:
                cs = str(c)
                list_to_print = [f + '\n' for f in cs.split('\\n')]
                f.writelines(list_to_print + ['\n'])
            else:
                f.writelines([str(c)] + ['\n'])
    elif type(content) is str:
        f.write(t + '\n' + content + '\n')
    f.close()


def command_executer(command, timeout, file):
    print("command: {}".format(str(command)))
    logger(file, " ".join(command))
    with subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env) as process:
        try:
            stdout, stderr = process.communicate(input, timeout=timeout)
        except subprocess.TimeoutExpired:
            process.kill()
            mesage = 'command: {} has been killed after timeout {}'.format(" ".join(command), timeout)
            print(mesage)
            stdout, stderr = process.communicate()
            logger(file, str(stdout))
            logger(file, str(stderr))
        except Exception:
            process.kill()
            process.wait()
            mesage = 'command: {} has been killed after timeout {}'.format(" ".join(command), timeout)
            print(mesage)
            logger(file, mesage)
            raise
        retcode = process.poll()
        logger(file, [process.args, retcode, stdout, stderr])
        if retcode and retcode != 254:
            return False
        else:
            return True


def main_pipeline(files):
    global SOURCE_PATH, SANDBOX_DIR, OUTPUTDIR, RERUN
    if RERUN:
        clean_dir(OUTPUTDIR)
    print("number of files: {}".format(len(files)))
    for i, f in enumerate(sorted(files)):
        start_time = time.time()
        print("{:.2f}".format(100 * i / len(files)), "%", f)
        print(i)
        ff = os.path.abspath(f)
        basename = os.path.basename(ff)
        dirname = os.path.dirname(ff)
        print("basename: {}".format(basename))
        print("dirname: {}".format(dirname))
        base_dirname = os.path.basename(dirname)
        dir_dirname = os.path.dirname(dirname)
        print("base_dirname: {}".format(base_dirname))
        print("dir_dirname: {}".format(dir_dirname))
        # Step 1: run SolidityTestGen.py -i f
        SolidityTestGen.main(f, TIMEOUT)
        # Step 2: mkdir: OUTPUTDIR + "/" + base_dirname
        new_sub_dir = OUTPUTDIR + "/" + base_dirname
        if not os.path.exists(new_sub_dir):
            os.mkdir(new_sub_dir)
        # Step 3. copy sanbox to new_dir_path
        new_dir_path = OUTPUTDIR + "/" + base_dirname + "/" + os.path.splitext(basename)[0]
        copy_dir(SANDBOX_DIR, new_dir_path)

        to_print_var = 'total time: {} seconds'.format(time.time() - start_time)
        #logger(os.path.dirname(f) + '/log.txt', to_print_var)


def main():
    start_time = time.time()
    init()
    global SOURCE_PATH, SANDBOX_DIR, OUTPUTDIR, RERUN, TIMEOUT
    parser = argparse.ArgumentParser(description='python script to run Sol Test Generation for all files in dir')
    insourse = ['-i', '--input_source']
    kwsourse = {'type': str, 'help': 'Input .sol-file. or directory with .sol-files'}

    # outdir = ['-o', '--output_dir']
    # kwoutdir = {'type': str, 'help': 'Output direcory name. Default: OUTPUTDIR = ../testgen_output.'}
    
    timeout = ['-t', '--timeout']
    kwtimeout = {'type': int, 'help': 'integer time in seconds. Default: 120'}

    parser.add_argument(*insourse, **kwsourse)
    # parser.add_argument(*outdir, **kwoutdir)
    parser.add_argument(*timeout, **kwtimeout)
    kwcov = {'type': bool, 'help': 'true - rerun / false - continue. Default: true.'}
    parser.add_argument('--rerun', **kwcov)

    args = parser.parse_args()

    RERUN = True
    if args.rerun is not None:
        if args.rerun == 'false':
            RERUN = False
        if args.rerun == 'true' or args.rerun == 'True':
            RERUN = True

    print(RERUN)

    files = []
    if args.input_source is not None:
        if os.path.isfile(args.input_source):
            file = args.input_source
            print('input file was set to {}'.format(file))
            files = [file]
        elif os.path.isdir(args.input_source):
            print('input directory was set to {}'.format(args.input_source))
            SOURCE_PATH = args.input_source
            files = sorted([os.path.join(dp, f) for dp, dn, filenames in os.walk(SOURCE_PATH)
                            for f in filenames if os.path.splitext(f)[1] == '.sol'
                            and os.path.splitext(f)[0] != "harness"])
            if not RERUN: # if rerun flase => find all finished files in testgen_output (-o: dir)
                destination = os.path.abspath(args.output_dir)
                subdirs = [f.path for f in os.scandir(destination) if f.is_dir() and os.path.basename(f)]
                al_run = []
                for sd in subdirs:
                    al_run += [f.path for f in os.scandir(sd) if f.is_dir() and os.path.basename(f)]
                al_run = [os.path.basename(a) + ".sol" for a in al_run]
                new_files = []
                for f in files:
                    if os.path.basename(f) not in al_run:
                        new_files.append(f)
                files = new_files # update all files and include only NOT fished file
        else:
            print('invalid input_source: {}'.format(args.input_source))
            exit(1)
    if args.timeout is not None:
        TIMEOUT = args.timeout
        print(f"timeout set to {TIMEOUT}")
    
    # if args.output_dir is not None:
    #     print('sandoutput dir set to {}'.format(args.output_dir))
    #     OUTPUTDIR = args.output_dir

    for f in files:
        print(f)
    main_pipeline(files)
    html_report.buildReport(OUTPUTDIR)
    # html_report.build_excel_report(OUTPUTDIR)
    clean_dir(SANDBOX_DIR)
    os.rmdir(SANDBOX_DIR)
    # html_report.buildReport_Excel_klee(SANDBOX_DIR)
    tt = time.time() - start_time
    to_print_var = 'total time: {} seconds'.format(tt)
    print(to_print_var)


if __name__ == "__main__":
    main()


import os.path
from datetime import datetime
import matplotlib.pyplot as plt

def read_log(filename):
    if not os.path.isfile(filename):
        return None
    lines = open(filename, 'r').readlines()
    results = []
    for line in lines:
        if 'rq_t' in line:
            tmp = line.strip().split()
            if len(tmp) >= 6:
                results.append(tmp[5])
    return results


def plot_ts(times, new_file_abs_path):
    result = [(t - times[i]).total_seconds() for i, t in enumerate(times[1:])]
    x = range(1, len(result) + 1)
    plt.clf()
    plt.plot(x, result)
    plt.xlabel("x - request to z3")
    plt.ylabel('y - time delta')
    plt.title('iteration progress')
    #plt.show()
    plt.savefig(new_file_abs_path)


def generate_plot(log_file, new_file_abs_path):
    times = read_log(log_file)
    print(times)
    try:
        plot_ts([datetime.strptime(t, "%H:%M:%S") for t in times], new_file_abs_path)
    except:
        print("An exception plot_ts")


if __name__ == '__main__':
    generate_plot('log.txt', 'img.png')